@@ -7,7 +7,8 @@ else
     export LD_LIBRARY_PATH="${CURRENT_DIR}/src/":${LD_LIBRARY_PATH}
 fi
 
-export PATH=$(pwd)/utils/nik2img/:${PATH}
+export PATH=$(pwd)/utils/mapnik-render/:${PATH}
+export PATH=$(pwd)/utils/mapnik-index/:${PATH}
 export PATH=$(pwd)/utils/mapnik-config/:${PATH}
 
 # mapnik-settings.env is an optional file to store
