@@ -33,17 +33,47 @@
 #include <mapnik/feature_layer_desc.hpp>
 #include <mapnik/value_types.hpp>
 
-// boost
+#pragma GCC diagnostic push
+#include <mapnik/warning_ignore.hpp>
 #include <boost/optional.hpp>
+#include <boost/version.hpp>
+#include <boost/geometry/index/rtree.hpp>
+#pragma GCC diagnostic pop
 
 // stl
 #include <vector>
 #include <deque>
 #include <string>
 
+template <std::size_t Max, std::size_t Min>
+struct csv_linear : boost::geometry::index::linear<Max,Min> {};
+
+namespace boost { namespace geometry { namespace index { namespace detail { namespace rtree {
+
+template <std::size_t Max, std::size_t Min>
+struct options_type<csv_linear<Max,Min> >
+{
+    using type = options<csv_linear<Max, Min>,
+                         insert_default_tag,
+                         choose_by_content_diff_tag,
+                         split_default_tag,
+                         linear_tag,
+#if BOOST_VERSION >= 105700
+                         node_variant_static_tag>;
+#else
+                         node_s_mem_static_tag>;
+
+#endif
+};
+}}}}}
+
 class csv_datasource : public mapnik::datasource
 {
 public:
+    using box_type = mapnik::box2d<double>;
+    using item_type = std::pair<box_type, std::pair<std::size_t, std::size_t>>;
+    using spatial_index_type = boost::geometry::index::rtree<item_type,csv_linear<16,4>>;
+
     csv_datasource(mapnik::parameters const& params);
     virtual ~csv_datasource ();
     mapnik::datasource::datasource_t type() const;
@@ -53,29 +83,27 @@ public:
     mapnik::box2d<double> envelope() const;
     mapnik::layer_descriptor get_descriptor() const;
     boost::optional<mapnik::datasource_geometry_t> get_geometry_type() const;
-    template <typename T>
-    void parse_csv(T & stream,
-                   std::string const& escape,
-                   std::string const& separator,
-                   std::string const& quote);
-
 private:
+    template <typename T>
+    void parse_csv(T & stream);
+    template <typename T>
+    boost::optional<mapnik::datasource_geometry_t> get_geometry_type_impl(T & stream) const;
+
     mapnik::layer_descriptor desc_;
     mapnik::box2d<double> extent_;
     std::string filename_;
-    std::string inline_string_;
-    unsigned file_length_;
     mapnik::value_integer row_limit_;
-    std::deque<mapnik::feature_ptr> features_;
-    std::string escape_;
-    std::string separator_;
-    std::string quote_;
+    std::string inline_string_;
+    char separator_;
+    char quote_;
     std::vector<std::string> headers_;
     std::string manual_headers_;
     bool strict_;
-    double filesize_max_;
     mapnik::context_ptr ctx_;
     bool extent_initialized_;
+    std::unique_ptr<spatial_index_type> tree_;
+    detail::geometry_column_locator locator_;
+    bool has_disk_index_;
 };
 
 #endif // MAPNIK_CSV_DATASOURCE_HPP
