@@ -28,21 +28,19 @@
 #include <mapnik/safe_cast.hpp>
 #include <mapnik/svg/svg_parser_exception.hpp>
 #include <mapnik/util/file_io.hpp>
+#include <mapnik/util/utf_conv_win.hpp>
 #include "agg_ellipse.h"
 #include "agg_rounded_rect.h"
 #include "agg_span_gradient.h"
 #include "agg_color_rgba.h"
 
 #pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wunused-parameter"
-#pragma GCC diagnostic ignored "-Wunused-local-typedef"
-#pragma GCC diagnostic ignored "-Wsign-conversion"
+#include <mapnik/warning_ignore.hpp>
 #include <boost/spirit/include/qi.hpp>
 #include <boost/spirit/include/phoenix_core.hpp>
 #include <boost/spirit/include/phoenix_operator.hpp>
 #include <boost/fusion/include/std_pair.hpp>
 #include <boost/algorithm/string/predicate.hpp>
-// rapidxml
 #include <boost/property_tree/detail/xml_parser_read_rapidxml.hpp>
 #pragma GCC diagnostic pop
 
@@ -156,6 +154,28 @@ double parse_double_optional_percent(T & error_messages, const char* str, bool &
     return val;
 }
 
+template <typename T>
+bool parse_double_list(T & error_messages, const char* str, double* list)
+{
+    using namespace boost::spirit::qi;
+    using boost::phoenix::ref;
+    qi::_1_type _1;
+    qi::double_type double_;
+    qi::lit_type lit;
+    using skip_type = boost::spirit::ascii::space_type;
+
+    if (!phrase_parse(str, str + std::strlen(str),
+                      double_[ref(list[0])=_1] >> -lit(',') >>
+                      double_[ref(list[1])=_1] >> -lit(',') >>
+                      double_[ref(list[2])=_1] >> -lit(',') >>
+                      double_[ref(list[3])=_1], skip_type()))
+    {
+        error_messages.emplace_back("failed to parse list of doubles from " + std::string(str));
+        return false;
+    }
+    return true;
+}
+
 bool parse_style (char const* str, pairs_type & v)
 {
     using namespace boost::spirit::qi;
@@ -464,16 +484,44 @@ void parse_dimensions(svg_parser & parser, rapidxml::xml_node<char> const* node)
 {
     double width = 0;
     double height = 0;
+    double aspect_ratio = 1;
+    double viewbox[4] = {0,0,0,0};
+    bool has_viewbox = false;
+    bool has_percent_height = true;
+    bool has_percent_width = true;
+
     auto const* width_attr = node->first_attribute("width");
     if (width_attr)
     {
-        width = parse_double(parser.error_messages_, width_attr->value());
+        width = parse_double_optional_percent(parser.error_messages_, width_attr->value(), has_percent_width);
     }
     auto const* height_attr = node->first_attribute("height");
     if (height_attr)
     {
-        height = parse_double(parser.error_messages_, height_attr->value());
+        height = parse_double_optional_percent(parser.error_messages_, height_attr->value(), has_percent_height);
     }
+    auto const* viewbox_attr = node->first_attribute("viewBox");
+    if (viewbox_attr)
+    {
+        has_viewbox = parse_double_list(parser.error_messages_, viewbox_attr->value(), viewbox);
+    }
+
+    if (has_percent_width && !has_percent_height && has_viewbox)
+    {
+        aspect_ratio = viewbox[2] / viewbox[3];
+        width = aspect_ratio * height;
+    }
+    else if (!has_percent_width && has_percent_height && has_viewbox)
+    {
+        aspect_ratio = viewbox[2] / viewbox[3];
+        height = height / aspect_ratio;
+    }
+    else if (has_percent_width && has_percent_height && has_viewbox)
+    {
+        width = viewbox[2];
+        height = viewbox[3];
+    }
+
     parser.path_.set_dimensions(width, height);
 }
 
@@ -974,7 +1022,11 @@ svg_parser::~svg_parser() {}
 
 bool svg_parser::parse(std::string const& filename)
 {
+#ifdef _WINDOWS
+    std::basic_ifstream<char> stream(mapnik::utf8_to_utf16(filename));
+#else
     std::basic_ifstream<char> stream(filename.c_str());
+#endif
     if (!stream)
     {
         std::stringstream ss;
