@@ -35,8 +35,7 @@
 #include <mapnik/svg/svg_path_attributes.hpp>
 
 #pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wunused-parameter"
-#pragma GCC diagnostic ignored "-Wunused-local-typedef"
+#include <mapnik/warning_ignore.hpp>
 #include <boost/algorithm/string.hpp>
 #include <boost/program_options.hpp>
 #pragma GCC diagnostic pop
@@ -108,7 +107,7 @@ struct main_marker_visitor
 #else
             s << "xdg-open " << png_name;
 #endif
-            int ret = system(s.str().c_str());
+            int ret = std::system(s.str().c_str());
             if (ret != 0)
                 status = ret;
         }
